/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "E:\SDK2_SE_R\SEMod_struct.h"
#include "E:\SDK2_SE_R\SEPin.h"

#include <assert.h>

#include <stdio.h> // files
#include <io.h>

#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <wchar.h>



void Module::sub_process(long buffer_offset, long sampleFrames )
{ 
	{
		CallHost(seaudioMasterSleepMode);
	}
}

// The core of a MIDI plugin
void Module::OnMidiData( unsigned long p_clock, unsigned long midi_msg, short pin_id)
{
 
	// system messages are for timing info etc
	// they don't have a channel, just pass them right through
	bool is_system_msg = ( midi_msg & SYSTEM_MSG ) == SYSTEM_MSG;

	if( !is_system_msg )
	{
		  if(thru == 1) 
		  {	  
		   	int chan = midi_msg & 0x0f;
		    int stat = midi_msg & 0xf0;

		// the next two bytes depend on the type of message
		    int b2 = ( midi_msg / 0x100 ) & 0xff;
		    int b3 = ( midi_msg / 0x10000 ) & 0xff;

		switch( stat )
		{
		case NOTE_ON:
		case NOTE_OFF:
			{
				 // byte 2 is the note number
				if( b2 <= transpose_ammount)
				{
                 midi_msg = stat + chan + (b2 << 8) + (b3 << 16);
                 getPin(PN_MIDI_OUT)->TransmitMIDI(SampleClock(), midi_msg);
				}
			//thisP = 0.5f + (float)( b2 - 69 )  / 12.0f;
			}  
		}
		      
			  
			  
			  
		  
		  }
	}
}
/*
		// The first byte of a midi message contains the status (note on/off etc) and the channel
		int chan = midi_msg & 0x0f;//channel
		int stat = midi_msg & 0xf0;//on off
		// the next two bytes depend on the type of message
		int b2 = ( midi_msg / 0x100 ) & 0xff; // note
		int b3 = ( midi_msg / 0x10000 ) & 0xff; // velo
        */
Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{   
	err2 = -1; err = -1;isnew = false;
	SEModule_base::open();
	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);
}

Module::~Module()
{
	_fcloseall();
}

// describe the module
bool Module::getModuleProperties (SEModuleProperties* properties)
{ 
   LL dish;
   BB bish;
   sprintf_s(dish.thisSEM   ,100, "%s\n",  "MIDI_ON_FILT");
   int j;

   j  = sprintf_s(  bish.buffer    ,1000, "%s\n", " ************************************" );
   j  = sprintf_s(  bish.buffer + j,1000, "%s\n", " * 2009 December in Tbingen GERMANY " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *            FAKE MUSIC             " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *         by ralph.a.cronin         " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s"  ,   dish.thisSEM );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *    RECORDER for SMIDI Files       " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *          enjoy SYNTHEDIT          " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *        by Jeff Mc Clintock        " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );

   // MAXIS :
   // j += sprintf( buffer + j, "%s\n", " **************************************************************************************" );//120
   // j += sprintf( buffer + j, "%s\n", "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" ); // = 54


	properties->name = dish.thisSEM;	// this is the name the end-user will see

	// return a uni+																																																			que string.
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug. No two plugs may have the same id.
	properties->id = dish.thisSEM;

	// Info, may include Author, Web page whatever
	properties->about =  bish.buffer;//"by ralph a. cronin and Jeff M";
	return true;
  
}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )
	{
	case 0:
		properties->name				= "MIDI In";
		properties->direction			= DR_IN;
		properties->datatype			= DT_MIDI2;
		break;
	case 1:	
		properties->name				= "ON";
		properties->variable_address	= &thru;
		properties->datatype_extra		= "range 0,1";
		properties->default_value		= "0";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		break;
    case 2:
		properties->name				= "filt_up";
		properties->variable_address	= &transpose_ammount;
		properties->datatype_extra		= "range 0,128";
		properties->default_value		= "60";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		break;
		
    case 3:
		properties->name				= "MIDI Out";
		properties->direction			= DR_OUT;
		properties->datatype			= DT_MIDI2;
		break;

	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

void Module::diag(char * melde)
{
 err2= fopen_s(&textfile,"D:\\DIAGNOSE.TXT","a+");
 if(err2 == 0)
 {
	 fwrite(melde, sizeof(char),strlen(melde), textfile);
     fclose(textfile);
 }
}

void Module::OnPlugStateChange(SEPin *pin)
{ 
	
}
/*
void Module::doaway()
{
     if(err == 0)
	 {
		 fclose( smidifileout);
		 last_take = 0;
	 }
     _unlink(smidi_filename );
     err =  fopen_s(&smidifileout, smidi_filename,"a+b");
}
void Module::loadarray()
{
     if(err == 0)
	 {
		 fclose(smidifileout);
		 last_take = 0;
	 }

     err =  fopen_s(&smidifileout, smidi_filename,"r+b");
	 if(err == 0)
	 {
      
	  int filehandle = _fileno(smidifileout);
      flang = _filelength(filehandle);
	  flang = flang / sizeof(char);
      
         
	  PLAY = false;
      if(tak_note != NULL) delete [] tak_note;
      tak_note = new char [++flang]; 

      unsigned long itaki = 0;
	  while(! feof(smidifileout))
      {
                fread(&ch,sizeof(char ),1,smidifileout);        // MIDI_msg
       tak_note[itaki]= ch;
                itaki++;                                      // next two    
	  }
      last_take = --itaki;
      
	  
      fclose(smidifileout);// THAT WAS THE MIDIFILE 
	  if(admin)
	  {
       sprintf_s(text,400,"%s itaki= %d\n",thisSEM, itaki);
	   diag(text);
	  }
     
	 }
	 REC = false;
     err =  fopen_s(&smidifileout, smidi_filename,"a+b");
}
*/

